// Copyright (C) 2024 Apple Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
// BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

import Foundation
public import SwiftUI
@_spi(CrossImportOverlay) import WebKit

extension WebPage {
    /// The theme color that the system gets from the first valid meta tag in the webpage.
    @available(WK_IOS_TBA, WK_MAC_TBA, WK_XROS_TBA, *)
    @available(watchOS, unavailable)
    @available(tvOS, unavailable)
    public var themeColor: Color? {
        self.backingProperty(\.themeColor, backedBy: \.themeColor) { backingValue in
            // The themeColor property is a UIColor/NSColor in WKWebView.
            #if canImport(UIKit)
            return backingValue.map(Color.init(uiColor:))
            #else
            return backingValue.map(Color.init(nsColor:))
            #endif
        }
    }

    /// Generates an image from the web view’s contents.
    ///
    /// - Parameter configuration: The object that specifies the portion of the web page to capture, and other capture-related behaviors.
    /// - Returns: An image that contains the specified portion of the webpage.
    /// - Throws: An error if a problem occurred when generating the snapshot.
    @available(WK_IOS_TBA, WK_MAC_TBA, WK_XROS_TBA, *)
    @available(watchOS, unavailable)
    @available(tvOS, unavailable)
    public func snapshot(_ configuration: WKSnapshotConfiguration = .init()) async throws -> Image? {
        let cocoaImage = try await backingWebView.takeSnapshot(configuration: configuration)

        #if canImport(UIKit)
        return Image(uiImage: cocoaImage)
        #else
        return Image(nsImage: cocoaImage)
        #endif
    }
}
